@@ -282,6 +282,9 @@ class TransmissionListener:
         except:
             pass
 
+    def __del__(self):
+        self.Terminate()
+
 
 def StartConversation(conversation_name,
                       peerID,
@@ -694,12 +697,15 @@ class Conversation:
             transmission_send_timeout_sec=transmission_send_timeout_sec,
             transmission_request_max_retries=transmission_request_max_retries)
 
-    def Close(self):
+    def Terminate(self):
         if self.listener:
             self.listener.Terminate()
 
+    def Close(self):
+        self.Terminate()
+
     def __del__(self):
-        self.Close()
+        self.Terminate()
 
 
 class ConversationListener:
@@ -754,6 +760,9 @@ class ConversationListener:
     def Terminate(self):
         self.listener.Terminate()
 
+    def __del__(self):
+        self.Terminate()
+
 
 def TransmitFile(filepath,
                  peerID,
@@ -977,6 +986,10 @@ class FileTransmitter:
         if info[0].decode('utf-8') == "ready":
             self.StartTransmission()
 
+    def __del__(self):
+        if self.conversation:
+            self.conversation.Close()
+
 
 class FileTransmissionReceiver:
     transmission_started = False
@@ -1071,6 +1084,9 @@ class FileTransmissionReceiver:
             else:
                 self.eventhandler(self.conv.peerID, filepath)
 
+    def __del__(self):
+        self.conv.Close()
+
 
 # ----------IPFS Technicalitites-------------------------------------------
 connections_send = list()
@@ -1083,14 +1099,10 @@ def CreateSendingConnection(peerID: str, protocol: str, port=None):
 
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     if port == None:
-        for prt in sending_ports:
-            try:
-                IPFS_API.ForwardFromPortToPeer(protocol, prt, peerID)
+        for prt in sending_ports:   # trying ports until we find a free one
+            if IPFS_API.ForwardFromPortToPeer(protocol, prt, peerID):
                 sock.connect(("127.0.0.1", prt))
-
                 return sock
-            except:
-                pass
 
         print("failed to find free port for sending connection")
     else:
@@ -1167,20 +1179,20 @@ def CloseListeningConnection(protocol, port):
         protocol=f"/x/{protocol}", targetaddress=f"/ip4/127.0.0.1/tcp/{port}")
 
 
-def ListenToBuffersOnPort(eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None):
-    return Listener(eventhandler, port, buffer_size, monitoring_interval, status_eventhandler)
-
-
-def ListenToBuffers(eventhandler, proto, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None, eventhandlers_on_new_threads=True):
-    listener = ListenerTCP(eventhandler, 0, buffer_size,
-                           eventhandlers_on_new_threads=eventhandlers_on_new_threads)
-    CreateListeningConnection(proto, listener.port)
-    return listener
-
-
-def SendBufferToPort(buffer, addr, port):
-    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-    sock.sendto(buffer, (addr, port))
+def ListenToBuffers(eventhandler,
+                    proto,
+                    buffer_size=def_buffer_size,
+                    monitoring_interval=2,
+                    status_eventhandler=None,
+                    eventhandlers_on_new_threads=True):
+    return BufferReceiver(
+        eventhandler,
+        proto,
+        buffer_size=buffer_size,
+        monitoring_interval=monitoring_interval,
+        status_eventhandler=status_eventhandler,
+        eventhandlers_on_new_threads=eventhandlers_on_new_threads
+    )
 
 
 class BufferSender():
@@ -1197,6 +1209,44 @@ class BufferSender():
             self.sock = CreateSendingConnection(self.peerID, self.proto)
             self.sock.send(data)
 
+    def Terminate(self):
+        CloseSendingConnection(self.peerID, self.proto)
+
+    def __del__(self):
+        self.Terminate()
+
+
+class BufferReceiver():
+    def __init__(self,
+                 eventhandler,
+                 proto,
+                 buffer_size=def_buffer_size,
+                 monitoring_interval=2,
+                 status_eventhandler=None,
+                 eventhandlers_on_new_threads=True):
+        self.proto = proto
+        self.listener = ListenerTCP(
+            eventhandler,
+            0,
+            buffer_size=buffer_size,
+            monitoring_interval=monitoring_interval,
+            status_eventhandler=status_eventhandler,
+            eventhandlers_on_new_threads=eventhandlers_on_new_threads
+        )
+        CreateListeningConnection(proto, self.listener.port)
+
+    def Terminate(self):
+        CloseListeningConnection(self.proto, self.listener.port)
+        self.listener.Terminate()
+
+    def __del__(self):
+        self.Terminate()
+
+
+def SendBufferToPort(buffer, addr, port):
+    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    sock.sendto(buffer, (addr, port))
+
 
 class ListenerTCP(threading.Thread):
     """
@@ -1208,7 +1258,8 @@ class ListenerTCP(threading.Thread):
             print("Received data from  " + sender_peerID)
             print(data.decode("utf-8"))
 
-        listener = Listener(eventhandler, 0, 2048)   # start listening to incoming buffers on an automatically assigned port (that's what the 0 means)
+        # start listening to incoming buffers on an automatically assigned port (that's what the 0 means)
+        listener = Listener(eventhandler, 0, 2048)
         port = listener.port    # retrieve the automatically assigned port
 
         # Once finished and listening on that port should be stopped:
@@ -1226,14 +1277,20 @@ class ListenerTCP(threading.Thread):
     sock = None
     last_time_recv = datetime.utcnow()
 
-    def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None, eventhandlers_on_new_threads=True):
+    def __init__(self,
+                 eventhandler,
+                 port=0,
+                 buffer_size=def_buffer_size,
+                 monitoring_interval=2,
+                 status_eventhandler=None,
+                 eventhandlers_on_new_threads=True):
         threading.Thread.__init__(self)
         self.port = port
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
         self.eventhandlers_on_new_threads = eventhandlers_on_new_threads
         self.sock = socket.socket(
-            socket.AF_INET, socket.SOCK_STREAM)      # For UDP
+            socket.AF_INET, socket.SOCK_STREAM)
 
         self.sock.bind(("127.0.0.1", self.port))
         # in case it had been 0 (requesting automatic port assiggnent)
@@ -1274,7 +1331,6 @@ class ListenerTCP(threading.Thread):
                     self.eventhandler(data)
         conn.close()
         self.sock.close()
-        CloseListeningConnection(str(self.port), self.port)
         if print_log_connections:
             print("Closed listener.")
 
@@ -1297,8 +1353,11 @@ class ListenerTCP(threading.Thread):
         if print_log_connections:
             print("terminating listener")
         self.terminate = True   # marking the terminate flag as true
+        # SendBufferToPort("end".encode(), "127.0.0.1", self.port)
         self.sock.close()
-        # SendBufferToPort(bytearray([0]),"",self.port) # to make the listener's buffer receiving while loop move forwards so that it realises it has o stop
+
+    def __del__(self):
+        self.Terminate()
 
 
 class Listener2TCP(threading.Thread):
@@ -1571,6 +1630,10 @@ class Listener2(threading.Thread):
         # SendBufferToPort(bytearray([0]),"127.0.0.1",self.port) # to make the listener's buffer receiving while loop move forwards so that it realises it has o stop
 
 
+def ListenToBuffersOnPort(eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None):
+    return Listener(eventhandler, port, buffer_size, monitoring_interval, status_eventhandler)
+
+
 def AddIntegrityByteToBuffer(buffer):
     # Adding an integrity byte that equals the sum of all the bytes in the buffer modulus 256
     # to be able to detect data corruption:
